#
# Must define MAIN in file that includes this (e.g., MAIN = l1.ltx)
# Can define GENFIGS to any extra figures that need to be generated
# and GENFIGSRC to any source
# Set PDF=pdf to use pdflatex

ifeq ($(PDF),pdf)
FIGEXT    = pdf
OUTEXT    = pdf
else
FIGEXT    = eps
OUTEXT    = dvi
endif

PWD	  := $(shell pwd)
BASE      := $(basename $(MAIN))
SRCEXT    := $(patsubst $(BASE).%,%,$(MAIN))
XFIGS     := $(wildcard *.fig)
DIAFIGS   := $(wildcard *.dia)
TEXFIGS   := $(XFIGS:.fig=.tex) # $(DIAFIGS:.dia=.tex)
BINFIGS   := $(XFIGS:.fig=.$(FIGEXT)) $(DIAFIGS:.dia=.$(FIGEXT)) $(GENFIGS)
STYLINK	  := $(notdir $(wildcard ../*.sty))
STYFILES  := $(sort $(STYLINK) $(wildcard *.sty))
TEXFILES  := $(sort $(MAIN) $(wildcard *.tex) $(TEXFIGS))
TEXDIST   := $(filter-out $(TEXFIGS), $(TEXFIGS))

JGRAPHS   := $(wildcard *.j)
GGRAPHS   := $(wildcard *.gnuplot)
GRAPHS    := $(JGRAPHS.j=.$(FIGEXT)) $(GGRAPHS:.gnuplot=.$(FIGEXT))

ifeq ($(PDF),pdf)
IMAGESRC  := $(wildcard figs/*.gif) $(wildcard figs/*.eps) 
else
IMAGESRC  := $(wildcard figs/*.gif) 
endif
IMAGEGEN  := $(addsuffix .$(FIGEXT), $(basename $(IMAGESRC)))
IMAGES    := $(sort $(IMAGEGEN) $(wildcard figs/*.$(FIGEXT)))
IMAGEDIST := $(filter-out $(IMAGEGEN), $(IMAGES))
# $(wildcard figs/*.odp) $(wildcard figs/*.ppt)

ifeq ($(PDF),pdf)
LATEX     = pdflatex \\nonstopmode\\input
else
LATEX     = latex \\nonstopmode\\input
#DVIPS     = dvips -j0 -G0 -t letter
DVIPS     = dvips
#PS2PDF    = GS_OPTIONS=-sPAPERSIZE=letter ps2pdf -sPAPERSIZE=letter
PS2PDF    = ps2pdf
endif

ifneq ($(origin NOADD),undefined)
ADD	  = : git add
else
ADD	  = git add
endif

all: $(BASE).ps $(BASE).pdf

ifeq ($(PDF),pdf)
%.pdf %.tex: %.fig
	fig2dev -L pdftex -p1 $< > $*.pdf
	fig2dev -L pdftex_t -p $*.pdf $< > $*.tex

%.pdf: %.eps
	perl -ne 'exit 1 unless /\n/' $< \
		|| perl -p -i -e 's/\r/\n/g' $<
	epstopdf --outfile=$@ $<

figs/%.pdf: figs/%.gif
	convert $< $@

%.ps: %.pdf
	pdftops $< $@

else
%.ps: %.dvi
	$(DVIPS) $< -o $@

%.eps %.tex: %.fig
	fig2dev -L pstex -p1 $< > $*.eps
	fig2dev -L pstex_t -p $*.eps $< > $*.tex
endif

%.eps: %.dia
	unset DISPLAY; dia -n -e $@ $<
%.tex: %.dia
	unset DISPLAY; dia -n -e $@ $<
%.eps: %.j
	@rm -f $@~
	jgraph $< > $@~
	mv -f $@~ $@
%.eps: %.gnuplot %.data
	gnuplot $*.gnuplot

$(STYLINK):
	rm -f $@
	ln -s ../$@ .

RERUN = egrep -q '(^LaTeX Warning:|\(natbib\)).* Rerun' $(basename $@).log
UNDEFINED = egrep -q '^(LaTeX|Package natbib) Warning:.* undefined' \
	$(basename $@).log

$(BASE).$(OUTEXT): \
%.$(OUTEXT): %.$(SRCEXT) $(TEXFILES) $(STYFILES) $(BINFIGS) $(GRAPHS) $(IMAGES)
	$(LATEX) $< || ! rm -f $@
	! $(UNDEFINED) || $(LATEX) $< || ! rm -f $@
	! $(RERUN) || $(LATEX) $< || ! rm -f $@
	! $(RERUN) || $(LATEX) $< || ! rm -f $@
ifeq ($(PDF),pdf)
	touch $*.dvi
	test ! -f .xpdf-running || xpdf -remote $(BASE)-server -reload
else

$(BASE).pdf: %.pdf: %.ps
	$(PS2PDF) $< $@
	test ! -f .xpdf-running || xpdf -remote $(BASE)-server -reload
endif

SHRINK = 0.36
ROW1 = .97h
ROW2 = .66h
ROW3 = .35h
COL1 = 0w
COL2 = 0.47w
NUPSPEC1 = 0R@$(SHRINK)($(COL1),$(ROW1))+1R@$(SHRINK)($(COL2),$(ROW1))
NUPSPEC2 = 2R@$(SHRINK)($(COL1),$(ROW2))+3R@$(SHRINK)($(COL2),$(ROW2))
NUPSPEC3 = 4R@$(SHRINK)($(COL1),$(ROW3))+5R@$(SHRINK)($(COL2),$(ROW3))
NUPSPEC = 6:$(NUPSPEC1)+$(NUPSPEC2)+$(NUPSPEC3)
$(BASE)-print.ps: $(BASE).ps ../Lecture.mk
	pstops -pletter '$(NUPSPEC)'  $< $@
%.pdf: %.ps
	$(PS2PDF) $< $@
print: $(BASE)-print.pdf

DIST = GNUmakefile $(TEXFILES) $(XFIGS) $(DIAFIGS) $(GENFIGSRC) \
	$(JGRAPHS) $(GGRAPHS) $(GGRAPHS:.gnuplot=:.data) \
	$(IMAGESRC) $(IMAGEDIST) $(STYFILES) $(EXTRADIST)
dist: $(BASE).tar.gz
$(BASE).tar.gz: $(DIST) ../Lecture.mk ../genindex
	tar -chzf $@ -C .. Lecture.mk genindex \
		$(patsubst %, $(notdir $(PWD))/%, $(DIST)) \
	|| ! rm -f $@

preview: $(BASE).pdf
	if test -f .xpdf-running; then			\
		xpdf -remote $(BASE)-server -quit;	\
		! test -f .xpdf-running || sleep 0.1;	\
		! test -f .xpdf-running || sleep 1;	\
	fi
	touch .xpdf-running
	(xpdf -remote $(BASE)-server -z 100 $(BASE).pdf; rm -f .xpdf-running) &

show: $(BASE).pdf
	xpdf -fullscreen $(BASE).pdf

osx: $(BASE).pdf
	open $(BASE).pdf

#EXTRAIGNORE = '*.aux' '*.bbl' '*.blg' '*.log' '*.dvi' '*.bak' '*.out'
ignore:
	rm -f .gitignore
	(for file in $(EXTRAIGNORE) \
			$(STYFILES) .xpdf-running \
			$(BASE).tar.gz \
			$(TEXFIGS) $(BINFIGS) $(GRAPHS) $(IMAGEGEN) \
			$(BASE).ps $(BASE).pdf $(BASE)-print.*; do \
		echo "/$$file"; \
	done) | sort > .gitignore
	$(ADD) .gitignore

DESTDIR=../../www/notes
install: $(BASE).pdf $(BASE).tar.gz $(BASE)-print.pdf $(EXTRAINSTALL)
	test -d $(DESTDIR) || mkdir -p $(DESTDIR)
	topic=`sed -ne 's/^Lecture Topic: *//p' $(BASE).log` \
		&& test -n "$$topic" \
		&& ../genindex $(DESTDIR)/index.html $(BASE) "$$topic" \
			$(EXTRAINSTALL)
	cp $^ $(DESTDIR)/
	$(ADD) $(patsubst %, $(DESTDIR)/%, $^) $(DESTDIR)/index.html

clean:
	rm -f $(BASE).ps $(BASE).pdf $(BASE)-print.* $(BASE).tar.gz
	rm -f $(TEXFIGS) $(BINFIGS) $(GRAPHS) $(IMAGEGEN) $(EXTRACLEAN)
	for sty in $(STYLINK); do \
		test ! -f ../$$sty || rm -f $$sty; \
	done
	rm -f *.dvi *.aux *.log *.bbl *.blg *.lof *.lot *.toc *.bak *.out
	rm -f *~ *.core core

always:
	@:

.PHONY: install clean all always ignore preview show osx dist print
